<?php declare(strict_types=1);

namespace EuKit\Base\ViewModels;

use Infomaker\Everyware\Support\Date;
use Infomaker\Everyware\Base\OpenContent\QueryBuilder;
use Infomaker\Everyware\Support\GenericPropertyObject;
use Infomaker\Everyware\Base\OpenContent\OpenContentProvider;

class ArchivePage extends Page
{
    /**
     * @var Date
     */
    private $startTime;

    /**
     * @var Date
     */
    private $endTime;

    public function __construct(int $year, int $month = null, int $day = null)
    {
        $targetDate = Date::create($year, $month !== 0 ? $month : null, $day !== 0 ? $day : null);

        $this->setTimeSpan($targetDate->isFuture() ? Date::now() : $targetDate, $month, $day);

        $propertyObject = new GenericPropertyObject();
        $propertyObject->fill([
            'title' => $this->getTitle(),
            'articles' => $this->getArticles(),
        ]);

        parent::__construct($propertyObject);
    }

    protected function getArticles(): array
    {
        $provider = $this->getProvider();
        $articles = $provider->queryWithRequirements();

        return array_map(function (\OcArticle $article) {
            return [
                'headline' => $article->get_value('headline'),
                'permalink' => $article->get_permalink(),
                'published' => Date::createFromOcString($article->get_value('pubdate'))->diffForHumans()
            ];
        }, $articles);
    }

    protected function getTitle(): string
    {
        return sprintf('Archive from: %s To: %s', $this->startTime->toDateString(), $this->endTime->toDateString());
    }

    protected function getProvider()
    {
        return OpenContentProvider::setup([
            'q' => $this->buildTimeSpanQuery('Pubdate'),
            'contenttypes' => ['Article'],
            'sort.indexfield' => 'Pubdate',
            'sort.Pubdate.ascending' => 'false',
            'limit' => 30
        ]);
    }

    private function setTimeSpan(Date $targetDate, $month = null, $day = null): void
    {
        $this->startTime = $targetDate;
        $this->endTime = $this->startTime->isToday() ? Date::now() : $this->startTime->copy()->endOfDay();

        if ($day) {
            $this->startTime->startOfDay();

            return;
        }

        if ($month) {
            $this->startTime->startOfMonth();

            if ( ! $this->startTime->isCurrentMonth()) {
                $this->endTime->endOfMonth();
            }

            return;
        }

        $this->startTime->startOfYear();

        if ( ! $this->startTime->isCurrentYear()) {
            $this->endTime->endOfYear();
        }
    }

    private function buildTimeSpanQuery($property): string
    {
        return QueryBuilder::dateSpanSearch(
            $property,
            $this->startTime->copy()->toOcString(),
            $this->endTime->copy()->toOcString()
        );
    }
}
